import os
import time

import pytest
from typer.testing import CliRunner

import goshawk.domain as domain
from goshawk.adapters.snowflake_adapter import snowflake
from goshawk.logging import LOG
from goshawk.main import app

runner = CliRunner()

epoch_time = int(time.time())
adapter = snowflake()
adapter.connect()


@pytest.mark.skipif("RUNALLTESTS" not in os.environ, reason="takes too long")
def test_model_deploy_into_env():
    LOG.debug("in test_model_deploy")
    domain.models.clear_state()
    sourcedb = "mydatabase"
    envname = "deploytest"  ## + str(epoch_time)
    db_name = f"{sourcedb}_{envname}"
    res = adapter.con.cursor().execute(f"create or replace database {db_name}")

    result = runner.invoke(app, ["model-deploy", "--db-env", envname])
    assert result.exit_code == 0
    LOG.debug("Getting tables")
    sql = f"""select * from {db_name}.information_schema.tables where table_schema!='INFORMATION_SCHEMA'"""
    res = adapter.con.cursor().execute(sql).fetchall()
    assert len(res) == 8


@pytest.mark.skipif("RUNALLTESTS" not in os.environ, reason="takes too long")
def test_model_deploy_into_env_w_mask():
    LOG.debug("in test_model_deploy")
    domain.models.clear_state()
    sourcedb = "mydatabase"
    envname = "deploytest"  ## + str(epoch_time)
    db_name = f"{sourcedb}_{envname}"
    res = adapter.con.cursor().execute(f"create or replace database {db_name}")

    result = runner.invoke(app, ["model-deploy", "--db-env", envname, "--mask", "mydatabase.schema_a"])
    assert result.exit_code == 0
    sql = f"""select * from {db_name}.information_schema.tables where table_schema!='INFORMATION_SCHEMA'"""
    res = adapter.con.cursor().execute(sql).fetchall()
    assert len(res) == 8
    # result = runner.invoke(app, ["model-deploy"])
    # assert result.exit_code == 0
    # result = runner.invoke(app, ["init-env",'--db-env',envname])
    # assert result.exit_code == 0
    result = runner.invoke(app, ["model-deploy", "--db-env", envname, "--mask", "mydatabase.schema_b"])
    assert result.exit_code == 0
    sql = f"""select * from {db_name}.information_schema.tables where table_schema!='INFORMATION_SCHEMA'"""
    res = adapter.con.cursor().execute(sql).fetchall()
    assert len(res) == 8


@pytest.mark.skipif("RUNALLTESTS" not in os.environ, reason="takes too long")
def test_model_deploy_no_env():
    LOG.debug("test_model_deploy_no_env")
    domain.models.clear_state()
    sourcedb = "mydatabase"
    db_name = sourcedb
    res = adapter.con.cursor().execute(f"create or replace database {db_name}")

    result = runner.invoke(app, ["model-deploy"])
    assert result.exit_code == 0
    sql = f"""select * from {db_name}.information_schema.tables where table_schema!='INFORMATION_SCHEMA'"""
    res = adapter.con.cursor().execute(sql).fetchall()
    assert len(res) == 5
