import os
import time

import pytest
from typer.testing import CliRunner

import goshawk.domain as domain
from goshawk import app_settings
from goshawk.adapters.snowflake_adapter import snowflake
from goshawk.logging import LOG
from goshawk.main import app

runner = CliRunner()


epoch_time = int(time.time())
adapter = snowflake()
adapter.connect()


@pytest.mark.skipif("RUNALLTESTS" not in os.environ, reason="takes too long")
def test_initenv():
    LOG.debug("in test_initenv------------------")
    domain.models.clear_state()
    sourcedb = "mydatabase"
    envname = "inittest_" + str(epoch_time)
    target_db = f"{sourcedb}_{envname}"
    res = adapter.con.cursor().execute(f"create database if not exists {sourcedb}")
    envname = "inittest_" + str(epoch_time)
    result = runner.invoke(app, ["init-env", "--db-env", envname])
    assert result.exit_code == 0
    res = adapter.con.cursor().execute(f"SHOW DATABASES LIKE '{target_db}'").fetchall()
    assert len(res) == 1
    res = adapter.con.cursor().execute(f"drop database if exists {target_db}")


@pytest.mark.skipif("RUNALLTESTS" not in os.environ, reason="takes too long")
def test_initenv2(monkeypatch):
    domain.models.clear_state()
    monkeypatch.setattr(app_settings, "MODELS_PATH", "tests/testcases/simple/models")
    sourcedb = "simpledb"
    envname = "inittest_" + str(epoch_time)
    target_db = f"{sourcedb}_{envname}"
    res = adapter.con.cursor().execute(f"create database if not exists {sourcedb}")
    envname = "inittest_" + str(epoch_time)
    result = runner.invoke(app, ["init-env", "--db-env", envname])
    assert result.exit_code == 0
    res = adapter.con.cursor().execute(f"SHOW DATABASES LIKE '{target_db}'").fetchall()
    assert len(res) == 1
    res = adapter.con.cursor().execute(f"drop database if exists {target_db}")
