import os
import time

import pytest
from typer.testing import CliRunner

import goshawk.domain as domain
from goshawk.adapters.snowflake_adapter import snowflake
from goshawk.logging import LOG
from goshawk.main import app

runner = CliRunner()

epoch_time = int(time.time())
adapter = snowflake()
adapter.connect()


@pytest.mark.skipif("RUNALLTESTS" not in os.environ, reason="takes too long")
def test_refresh_single_schema_in_env():
    LOG.debug("in test_refresh_of_env")
    domain.models.clear_state()
    sourcedb = "mydatabase"
    envname = "deploytest"  ## + str(epoch_time)
    db_name = f"{sourcedb}_{envname}"
    res = adapter.con.cursor().execute(f"create or replace database {db_name}")
    result = runner.invoke(app, ["model-deploy", "--db-env", envname])
    assert result.exit_code == 0, "Deploy did not succeed"
    sql = f"""select * from {db_name}.information_schema.tables where table_schema!='INFORMATION_SCHEMA'"""
    res = adapter.con.cursor().execute(sql).fetchall()
    assert len(res) == 5, "Expected 5 tables"
    sql = f"""truncate table {db_name}.schema_a.model_a2"""
    # res = adapter.con.cursor().execute(sql).fetchall()
    res = adapter.execute_sql(sql).fetchall()
    result = runner.invoke(app, ["data-refresh", "--db-env", envname, "--mask", "mydatabase.schema_a"])
    assert result.exit_code == 0, "Refresh did not succeed"


@pytest.mark.skipif("RUNALLTESTS" not in os.environ, reason="takes too long")
def test_refresh_single_schema_in_prod():
    LOG.debug("in test_refresh_of_prod")
    domain.models.clear_state()
    sourcedb = "mydatabase"

    db_name = f"{sourcedb}"
    res = adapter.con.cursor().execute(f"create or replace database {db_name}")
    result = runner.invoke(app, ["model-deploy"])
    assert result.exit_code == 0, "Deploy did not succeed"
    sql = f"""select * from {db_name}.information_schema.tables where table_schema!='INFORMATION_SCHEMA'"""
    res = adapter.con.cursor().execute(sql).fetchall()
    assert len(res) == 5, "Expected 5 tables"
    sql = f"""truncate table {db_name}.schema_a.model_a2"""
    res = adapter.con.cursor().execute(sql).fetchall()
    result = runner.invoke(app, ["data-refresh", "--mask", "mydatabase.schema_a"])
    assert result.exit_code == 0, "Refresh did not succeed"
    sql = f"""select * from {db_name}.schema_a.model_a2"""
    rows = adapter.con.cursor().execute(sql).fetchall()
    assert len(rows) == 1
