from typer.testing import CliRunner

import goshawk.domain as domain
from goshawk import app_settings
from goshawk.main import app

runner = CliRunner()


def test_view_model_tree():
    result = runner.invoke(app, ["view-model-tree", "--dry-run"])
    assert result.exit_code == 0
    assert len(result.output.split(";")) == 8


def test_view_model_tree_with_mask():
    domain.models.clear_state()
    result = runner.invoke(app, ["view-model-tree", "--mask", "mydatabase.schema_a", "--dry-run"])
    assert result.exit_code == 0
    assert len(result.output.split(";")) == 4


def test_view_model_tree_schemas_only():
    domain.models.clear_state()
    result = runner.invoke(app, ["view-model-tree", "--schemas-only", "--dry-run"])
    assert result.exit_code == 0
    assert len(result.output.split(";")) == 4


def test_view_model_tree_simpledb_schemas_only(monkeypatch):
    domain.models.clear_state()
    monkeypatch.setattr(app_settings, "MODELS_PATH", "tests/testcases/simple/models")
    result = runner.invoke(app, ["view-model-tree", "--schemas-only", "--dry-run"])
    assert result.exit_code == 0
    assert "SIMPLEDB" in result.output
    assert len(result.output.split(";")) == 4


def test_view_model_tree_simpledb(monkeypatch):
    domain.models.clear_state()
    monkeypatch.setattr(app_settings, "MODELS_PATH", "tests/testcases/simple/models")
    result = runner.invoke(app, ["view-model-tree", "--dry-run"])
    assert result.exit_code == 0
    assert "SIMPLEDB" in result.output
    assert len(result.output.split(";")) == 11
